class PcCaptureSession
{
  static initialize(fOnLoad=null)
  {
    PcCaptureSession.shots = [];

    PcCaptureSession.currentShot = 0;
    PcCaptureSession.numShots = 0;

    PcCaptureSession.idShot = null;
    PcCaptureSession.idFrame = null;

    PcCaptureSession.defaultShots = [];
    PcCaptureSession.defaultCurrentShot = 0;
    PcCaptureSession.defaultNumShots = 0;

    PcCaptureSession.resources = [];

    PcCaptureSession.intializeResources(()=>{
      PcCaptureSession.loadDefaults();
      if (fOnLoad) fOnLoad();
    });

    PcCaptureSession.framing = new PcFraming();
    PcCaptureSession.framing.initialize();
  }

  static intializeResources(fOnLoad=null)
  {
    let urlsToLoad = [];
    let k=0;

    let testLoading = (url)=>{
        for (let i=0; i<urlsToLoad.length; i++) {if (urlsToLoad[i].url==url) urlsToLoad[i].loaded = true;}
        for (let i=0; i<urlsToLoad.length; i++) {if (urlsToLoad[i].loaded==false) return;}
        if (fOnLoad) fOnLoad();
    }

    urlsToLoad.push({name:"filter_general_vintagecover", url:PcConfig.host + "ressources/filter_general_vintagecover.png", loaded:false});
    urlsToLoad.push({name:"filter_general_whiteglow", url:PcConfig.host + "ressources/filter_general_whiteglow.png", loaded:false});
    urlsToLoad.push({name:"filter_general_yellowglow", url:PcConfig.host + "ressources/filter_general_yellowglow.png", loaded:false});

    if (PcConfig.scenario.mode=="photobooth")
    {
      urlsToLoad.push({name:"defaults_f1", url:PcConfig.host + "ressources/booth_frame1.jpg?x="+Math.random(), loaded:false}); // la meuf par défaut 1 2 3 4
      urlsToLoad.push({name:"defaults_f2", url:PcConfig.host + "ressources/booth_frame2.jpg?x="+Math.random(), loaded:false});
      urlsToLoad.push({name:"defaults_f3", url:PcConfig.host + "ressources/booth_frame3.jpg?x="+Math.random(), loaded:false});
      urlsToLoad.push({name:"defaults_f4", url:PcConfig.host + "ressources/booth_frame4.jpg?x="+Math.random(), loaded:false});
    }
    else {
      urlsToLoad.push({name:"defaults_f1", url:PcConfig.host + "ressources/kiosk_frame1.jpg?x="+Math.random(), loaded:false});
      urlsToLoad.push({name:"defaults_f2", url:PcConfig.host + "ressources/kiosk_frame2.jpg?x="+Math.random(), loaded:false});
      urlsToLoad.push({name:"defaults_f3", url:PcConfig.host + "ressources/kiosk_frame3.jpg?x="+Math.random(), loaded:false});
      urlsToLoad.push({name:"defaults_f4", url:PcConfig.host + "ressources/kiosk_frame4.jpg?x="+Math.random(), loaded:false});
    }

    for (let i=0; i<urlsToLoad.length; i++) {PcCaptureSession.loadResource(urlsToLoad[i].name, urlsToLoad[i].url, testLoading);}

    if (urlsToLoad.length==0) {if (fOnLoad) fOnLoad();}
  }

  static loadDefaults()
  {
    this.addDefaultFrame(PcCaptureSession.getResource("defaults_f1"), true);
    this.nextDefaultShot();

    this.addDefaultFrame(PcCaptureSession.getResource("defaults_f2"), true);
    this.nextDefaultShot();

    this.addDefaultFrame(PcCaptureSession.getResource("defaults_f3"), true);
    this.nextDefaultShot();

    this.addDefaultFrame(PcCaptureSession.getResource("defaults_f4"), true);

    // setTimeout(()=>this.filterShots(PcConfig.screen.format.filter, /*useDefaultShots=*/ true), 2000); //now done on demand via PcCaptureSessionFrame.getFiltered()
  }

  static nextShot()
  {
    if (PcCaptureSession.shots[PcCaptureSession.currentShot])
    {
        PcCaptureSession.currentShot++;
    }
  }

  static nextDefaultShot()
  {
    if (PcCaptureSession.defaultShots[PcCaptureSession.defaultCurrentShot])
    {
        PcCaptureSession.defaultCurrentShot++;
    }
  }

  static addFrame(data, dataIsCanvas=false)
  {
    if (!PcCaptureSession.shots[PcCaptureSession.currentShot])
    {
      PcCaptureSession.shots[PcCaptureSession.currentShot] = new PcCaptureSessionShot();
      PcCaptureSession.numShots++;
    }

    PcCaptureSession.shots[PcCaptureSession.currentShot].addFrame(data, dataIsCanvas);

    if (PcCaptureSession.currentShot == 0)
    {
      if (PcCaptureSession.shots[0].frames.length == 1) // addFrame: if shot has only one frame...
      {
        this.idFrame = new PcCaptureSessionFrame(data, dataIsCanvas); // addFrame: ... then make idFrame this very frame // WARNING: this frame is never to be removeBged online like the regular shots.frames.
      }
    }

  }

  static addDefaultFrame(data, dataIsCanvas=false)
  {
    if (!PcCaptureSession.defaultShots[PcCaptureSession.defaultCurrentShot])
    {
      PcCaptureSession.defaultShots[PcCaptureSession.defaultCurrentShot] = new PcCaptureSessionShot();
      PcCaptureSession.defaultNumShots++;
    }

    PcCaptureSession.defaultShots[PcCaptureSession.defaultCurrentShot].addFrame(data, dataIsCanvas);
  }

  /* applyFraming now takes predefined shots_landmarks
   if applicable they will be reused during the final call of PcFraming.apply()
   to avoid recomputing landmarks detection
  */
  static async applyFraming(onApplyFraming, predef_shots_landmarks, faceRatio, centerRatio, frameSize, error) // applies to PcCaptureSession.shots[] array
  {
    var shots_landmarks = [];
    let cur_predef_shot_landmarks;
   
    PcDbg.log(FRAMINGDBG, "PCS: applyFraming: predef_shots_landmarks", JSON.stringify(predef_shots_landmarks));
    PcDbg.log(FRAMINGDBG, "PCS: applyFraming: faceRatio",  faceRatio, "centerRatio", JSON.stringify(centerRatio), "frameSize", JSON.stringify(frameSize), "au mieux",  !alert);
  
    // call the PcCaptureSessionShot framing for each shot
    for (let i=0; i<PcCaptureSession.shots.length; i++)  {
      // ensure cur_predef_shot_landmarks sanity
      if (predef_shots_landmarks == null || predef_shots_landmarks.length < i+1) { /* fixme later: hack for predef_shots_landmarks sizing vs unpredictible frame.length! */
        cur_predef_shot_landmarks = null;
      } else {
        cur_predef_shot_landmarks = predef_shots_landmarks[i];
      }
      PcDbg.log(FRAMINGDBG, "PCS: applyFraming: -> cur_predef_shot_landmarks", i, "=", JSON.stringify(cur_predef_shot_landmarks));
      //accumulate the resulting shot landmark
      shots_landmarks.push(
        await PcCaptureSession.shots[i].framing(
          PcCaptureSession.framing /* it's a class instance ! bad inheritance! yuk!*/,
          cur_predef_shot_landmarks,
          faceRatio,
          centerRatio,
          frameSize,
          error)
      );
      PcDbg.log(FRAMINGDBG, "PCS: applyFraming -> shots_landmarks are now", JSON.stringify(shots_landmarks));
    }
    //call applyFraming callback aka onApplyFraming with the resulting shots_lanmarks -> this will speed up any call to makeID()
    onApplyFraming(shots_landmarks);

    return shots_landmarks;
  }

  static async createIdShot(onApplyFraming, predef_frame_landmarks, faceRatio, centerRatio, frameSize) // applies to PcCaptureSession.idFrame !
  {
    //PcCaptureSession.idShot = new PcCaptureSessionShot();
    //PcCaptureSession.idShot.addFrame(PcCaptureSession.idFrame.canvas, true);
    PcDbg.log(FRAMINGDBG, "PCS: createIdShot: predef_frame_landmarks", JSON.stringify(predef_frame_landmarks));
    PcDbg.log(FRAMINGDBG, "PCS: createIdShot: faceRatio",  faceRatio, "centerRatio", centerRatio, "frameSize", JSON.stringify(frameSize));
    
    await PcCaptureSession.idFrame.framing(PcCaptureSession.framing, predef_frame_landmarks, faceRatio, centerRatio, frameSize, true);

    onApplyFraming();
  }

  static _getShots(useDefaultShots)
  {
    return useDefaultShots ? PcCaptureSession.defaultShots : PcCaptureSession.shots;
  }
  static async removeBgShots(timeout, useDefaultShots)
  {
    let shots = this._getShots(useDefaultShots);
    let ret = (shots.length > 0);
    PcDbg.log(REMOVEDBG, "initially ret ", ret)

    await Promise.all(
      shots.map(async (shot) =>
        {
          ret &= await shot.removeBgShot(timeout);
        }
      )
    );
    PcDbg.log(REMOVEDBG, "returned ret", ret)
    return ret;
  }
  /*
  static filterShots(filter_dict, useDefaultShots=false)
  {
    if(false) {
      let shots = this._getShots(useDefaultShots);
    
      // forEach((shot)) iterates the list item,
      // forEach((shot, index)) also iterates the list item index
    
      shots.forEach((shot, index) => shot.filterShot(filter_dict, index)); // now done on demand by PcCaptureSessionFrame.getFiltered() 
   }  
  }
  */
  static reset()
  {
    PcCaptureSession.shots.forEach(shot => shot.delete());
    PcCaptureSession.shots.splice(0);
    PcCaptureSession.currentShot = 0;
    PcCaptureSession.numShots = 0;
  }

  static resetDefault()
  {
    PcCaptureSession.defaultShots.forEach(shot => shot.delete());
    PcCaptureSession.defaultShots.splice(0);
    PcCaptureSession.defaultCurrentShot = 0;
    PcCaptureSession.defaultNumShots = 0;
  }

  static loadResource(name, imgUrl, fOnLoad=null)
  {
    if (imgUrl=="") return;

    PcCaptureSession.resources[name] = new Image();

    if (fOnLoad)
    {
      PcCaptureSession.resources[name].onload = ()=>{fOnLoad(imgUrl)};
      PcCaptureSession.resources[name].onerror = ()=>{fOnLoad(imgUrl)};
    }

    PcCaptureSession.resources[name].src = imgUrl;
  }

  static getResource(name)
  {
    let img = PcCaptureSession.resources[name];
    let canvas = document.createElement('canvas');
    canvas.width = img.naturalWidth;
    canvas.height = img.naturalHeight;
    canvas.getContext('2d').drawImage(img, 0, 0);
    return canvas;
  }

}
