const RBG_FLAG_DONE=(1<<0);
const RBG_FLAG_SUCCESS=(1<<1);
const FRAMEDBG=false;
const RACEDBG=false;
class PcCaptureSessionFrame
{
  async loadImage(image, url) {
    return new Promise((resolve, reject) => {
      image.onload = () => resolve(image);
      image.onerror = reject;
      image.src = url;
    });
  }

  async prepareFrame(data, canvas, dataIsCanvas, isRemoveBg=false /* this arg is just for debug, remove it when bug #26 is finalized*/)
  {
    PcDbg.log(CRASHDBG|REMOVEDBG|FRAMEDBG, "entering with dataIsCanvas", dataIsCanvas, "isRemoveBg", isRemoveBg);
    if (dataIsCanvas)
    {
      let context = canvas.getContext('2d');
      canvas.width = data.width;
      canvas.height = data.height;
      context.drawImage(data, 0, 0, data.width, data.height);

      if (PcConfig.scenario.cameraOrientation=="portrait" || PcConfig.scenario.cameraOrientation=="portraitInverted")
      {
        let destW = canvas.width;
        let destH = canvas.height;

        let newcanvas = PcConfig.cloneCanvas(canvas);

        context.clearRect(0,0,canvas.width,canvas.height);
        canvas.width = destH;
        canvas.height = destW;
        context.save();
        context.translate(canvas.width/2, canvas.height/2);

        if (PcConfig.scenario.cameraOrientation=="portrait")
        {
          context.rotate(-90*Math.PI/180);
        }
        else {
          context.rotate(90*Math.PI/180);
        }

        context.drawImage(newcanvas, -destW/2, -destH/2, destW, destH);
        context.restore();
      }
      
    }
    else { // it is a file
      this.imageURL = data;
    
      PcDbg.log(CRASHDBG|REMOVEDBG|FRAMEDBG, "get image from url", this.imageURL, "with isRemoveBg", isRemoveBg);
      let image = document.createElement('img');

      PcDbg.log(RACEDBG, "***WAIT FOR LOADIMAGE", this.imageURL);

      /* then() ensures that the promise is settled, the first argument is the callback of success/fullfil settlement, the second argument is the callback of failure (none here) */

      await this.loadImage(image, this.imageURL) /*we wait for the promise: */ 
      .then( (image) => { /* promise fulfill callback *//* and then we collect the results */
          PcDbg.log(RACEDBG, "***CALL IMAGELOADED", this.imageURL);
          this.imageLoaded(canvas, image, isRemoveBg);
        }
      );

      // other way to do it (unclear how to get the failure settlement):
      /*
        image = await this.loadImage(image, this.imageURL);
        PcDbg.log(RACEDBG, "***CALL IMAGELOADED", this.imageURL);
        this.imageLoaded(canvas, image, isRemoveBg);
      */
      /* 
        WARNING: the following legacy code would give us a race condition where imageLoaded was called *AFTER* onRemoveBgDone 
        => we would end up with a default black canvas of 300x150 instead of the picture with transparent background
      */
      /*
      image.onload = () => this.imageLoaded(canvas, image, isRemoveBg); //  THIS IS ASYNCHRONOUS
      image.src = data; // pure js: actually loads the image and calls onload afterwards 
      */
    }

    PcDbg.log(CRASHDBG|REMOVEDBG|FRAMEDBG, "exiting");

  }

  constructor(data, dataIsCanvas=false)
  {
    this.canvas = document.createElement('canvas');
    this.canvas_removebg = document.createElement('canvas');
    this.filtered = {}; //dict that will contain one frame per applied filter
    this.filtered_removebg = {};
    this.removebg_status = 0;
    this.maxSourceWidth = 10000;
    PcDbg.log(FRAMEDBG, "dataIsCanvas:", dataIsCanvas, "data", data);
    this.prepareFrame(data, this.canvas, dataIsCanvas);
  }

  imageLoaded(canvas, image, isRemoveBg=false)
  {
    PcDbg.log(CRASHDBG|FRAMEDBG, "entering with isRemoveBg", isRemoveBg);
    var w = image.naturalWidth;
    var h = image.naturalHeight;
    var destW, destH;

    if (w>this.maxSourceWidth)
    {
      destW = this.maxSourceWidth;
      destH = destW * h/w;
    }
    else {
      destW = w;
      destH = h;
    }
    let context = canvas.getContext('2d');

    canvas.width = destW;
    canvas.height = destH;
    context.drawImage(image, 0, 0, destW, destH);

    if (PcConfig.scenario.cameraOrientation=="portrait" || PcConfig.scenario.cameraOrientation=="portraitInverted")
    {
      let newcanvas = PcConfig.cloneCanvas(canvas);

      context.clearRect(0,0,canvas.width,canvas.height);
      canvas.width = destH;
      canvas.height = destW;
      context.save();
      context.translate(canvas.width/2, canvas.height/2);

      if (PcConfig.scenario.cameraOrientation=="portrait")
      {
        context.rotate(-90*Math.PI/180);
      }
      else {
        context.rotate(90*Math.PI/180);
      }

      context.drawImage(newcanvas, -destW/2, -destH/2, destW, destH);
      context.restore();
    }

    if(SAVEDBG && isRemoveBg) { // see? isRemoveBg is just for debug
      
      PcConfig.saveCanvasToFile(canvas, this.imageURL+".rbg_save.png", "png");
    }
    PcDbg.log(CRASHDBG|FRAMEDBG, "exiting", image.src, "(check if I exited after prepareFrame)");
  }

  async onRemoveBgDone(out_filename)
  {
    this.removebg_status |= RBG_FLAG_DONE;
    this.removebg_status &= ~RBG_FLAG_SUCCESS;
    PcDbg.log(REMOVEDBG, "entering onRemoveBgDone with filename", out_filename);

    if (!out_filename) { // it didn't work
      PcDbg.log(REMOVEDBG, "no output filename, it seems it didn't work!");      
    } else {
      this.removebg_status |= RBG_FLAG_SUCCESS;
      PcDbg.log(REMOVEDBG, "about to prepare RBG CANVAS frame with URL", out_filename);
      if (0) {
        this.canvas_removebg = PcConfig.cloneCanvas(this.canvas); // THIS PROVES this.canvas IS OK, WE GET THE SAME PICTURES IN RBG/NONRBG UI SELECTOR

      } else {
        await this.prepareFrame(/*data=*/out_filename, /*canvas=*/this.canvas_removebg, /*dataIsCanvas=*/false,/*isRemoveBg=*/true);
        PcDbg.log(RACEDBG, "***EXITING PREPAREFRAME (MAKE SURE WE WAITED FOR PREPAREFRAME UNTIL EXIT)!");
      }
    }

    PcDbg.log(REMOVEDBG, "exiting with removebg_status is", this.removebg_status, "(check that prepareFrame ImageLoaded happened correctly)");
    return this.removebg_status;
  }

  isRemoveBgDone()
  {
    return (this.removebg_status & RBG_FLAG_DONE) != 0;
  }
  isRemoveBgSuccess()
  {
    return (this.removebg_status & RBG_FLAG_SUCCESS) != 0;
  }
  getFiltered(filter_dict, variation, useRemoveBg=false) // get a filtered frame, compute it if not available
  {
    let rbg_cond = useRemoveBg && this.isRemoveBgSuccess();
    let filtered = rbg_cond ? this.filtered_removebg : this.filtered;
    

    if (! filtered.hasOwnProperty([filter_dict.id, variation])) { // filter dict already contins the filtered picture -> don't do it anymore
      this.filterFrame(filter_dict, variation, useRemoveBg);
    }
 
    return filtered[[filter_dict.id,variation]];
  }
  async removeBgFrame(timeout)
  {
    PcDbg.log(REMOVEDBG, "entering with timeout", timeout);
    if (this.isRemoveBgDone()) {
      PcDbg.log(REMOVEDBG, "removebg already done, leaving");
      return;
    }
    if (timeout == undefined) {
      PcDbg.log(REMOVEDBG, "no timeout value asked for a Frame (== interactively on UI), we are failing, leaving");
      return await this.onRemoveBgDone(undefined); 
    }
    let in_filename;
    
   
    /*
      Note: from a canvas (webcam or nothing, no dslr, no webcam, nada), the picture has no URL -
      we create a file to give to removebg 
      -> either with classic php post data upload,
      or with fs.writeFileSync https://www.geeksforgeeks.org/node-js-fs-writefilesync-method/?ref=lbp
      SEE HERE:https://stackoverflow.com/questions/5867534/how-to-save-canvas-data-to-file

    */
    if(this.imageURL) {
      in_filename = this.imageURL;
    } else {
      let fs = require('electron').remote.require("fs");

      var uniqid= PcConfig.getUniqId();
      
      var name = "frame" + uniqid + ".jpg";
      in_filename = PcConfig.global.removeBgInDir + name;
      PcDbg.log(REMOVEDBG, "with rbgindir", PcConfig.global.removeBgInDir, "in_filename is", in_filename);
      fs.exists(PcConfig.global.removeBgInDir, exists =>
        {
          if (!exists) {
            fs.mkdirSync(PcConfig.global.removeBgInDir);
          }
        }
      );
 
      PcConfig.saveCanvasToFile(this.canvas, in_filename); // will be a jpeg
    }
    PcDbg.log(REMOVEDBG, "this is our context: ", JSON.stringify(this));
    let out_filename = PcRemoveBg.queueJob(in_filename,
      /*is_id=*/false,
      /*ttl_seconds=*/timeout,
      /*onRemoveBgDone=*/ undefined, //this.onRemoveBgDone, 
      /*id_photo=*/null,
      /*outFormat=*/ 'png',
      /*in_bg_filename=*/undefined
      );

      await this.onRemoveBgDone(out_filename);

      PcDbg.log(REMOVEDBG, "exiting", );
      PcDbg.log(RACEDBG, "***EXITING, MAKE SURE ONREMOVEBGDONE WAS DONE");
  }
  
  
  filterFrame(filter_dict, variation, useRemoveBg=false)
  {
    let filtered;
    let canvas;
    let mode;
    // returns a filtered canvas from PcFilter switch-case filtering code depending on filter name
    
    if (useRemoveBg && this.isRemoveBgSuccess()) { 
      mode = "rbg";
      canvas = this.canvas_removebg;
      filtered = this.filtered_removebg;
    } else {
      mode = "normal";
      canvas = this.canvas;
      filtered = this.filtered;
    }
    if (filtered.hasOwnProperty([filter_dict.id, variation])) { // filter dict already contins the filtered picture -> don't do it anymore
      return;
    }

    let fd;
    if (filter_dict.id) {
      fd = filter_dict.filterName.toString(); //id is a combination of filtername and args with lots of spaces in between and ":" separator before that
    } else { 
      fd = "undefined";
    }
   

    //PcDbg.log(REMOVEDBG|CRASHDBG, "variation", variation);
    //PcDbg.log(REMOVEDBG|CRASHDBG, "fd", fd);

    /* 
    debug stuff: dump removebg original canvas, just to make sure we have it (alpha png)
    note: we will have several copies of the same original alpha canvas! this is expected
    */
    /*if (SAVEDBG && useRemoveBg) {
      let filename = PcConfig.global.tmpDir + "pic_" + mode + "_" + fd + "_" + variation.toString() + "_ORI.png";
      PcDbg.log(REMOVEDBG|CRASHDBG, "saving original canvas of size", canvas.width, canvas.height, "to filename", filename, "png");
      PcConfig.saveCanvasToFile(canvas, filename, "png");
    }
    */
    var filter = new PcFilter(filter_dict);
    filtered[[filter_dict.id, variation]] = filter.apply(canvas, variation);
    
    // for debug: dump removebg filtered canvas
    if (SAVEDBG && useRemoveBg) {
      let filename = PcConfig.global.tmpDir + "pic_" + mode + "_" + fd + "_" + variation.toString() + "_FILTERED.png";
      PcDbg.log(FRAMEDBG|CRASHDBG, "saving filtered canvas to filename", filename);
      PcConfig.saveCanvasToFile(filtered[[filter_dict.id, variation]], filename, "png");
    }


  }

  /* per Frame framing now takes predef_frame_landmarks.
  it is used to potentially avoid recomputing landmarks 
  */
  async framing(framing, predef_frame_landmarks, faceRatio, centerRatio, frameSize, alert) // FIXME, handle framing on removebg_canvas
  {
    PcDbg.log(FRAMINGDBG, "PCSF: framing: predef_frame_landmarks", JSON.stringify(predef_frame_landmarks));
    PcDbg.log(FRAMINGDBG, "PCSF: framing: faceRatio",  faceRatio, "centerRatio", JSON.stringify(centerRatio), "frameSize", JSON.stringify(frameSize), "au mieux",  !alert);
    let frame_landmarks;
    [ this.canvas, frame_landmarks ]   = await framing.apply(
      predef_frame_landmarks, 
      this.canvas,
      faceRatio,
      centerRatio,
      frameSize,
      alert); // framing is a class instance from PcCaptureSession, yuk

      PcDbg.log(FRAMINGDBG, "PCSF: framing -> frame_landmarks =", JSON.stringify(frame_landmarks));

      // return the Frame landmarks
      return frame_landmarks;
  }

  
  delete()
  {
    for (var c of [this.canvas, this.canvas_removebg]) { // "var. of" works like "x in"  in python .. while "var. in" gives index of an array  
      c = null;
    }
    PcDbg.log(FRAMEDBG|REMOVEDBG, "filtered:", JSON.stringify(this.filtered), "filtered_removebg:", JSON.stringify(this.filtered_removebg));
    for (var filtered of [this.filtered, this.filtered_removebg]) 
    {
      for (var p in filtered) { // filtered is a dict, var..in is required to iterate on keys
       if (filtered.hasOwnProperty(p))
       {
          filtered[p] = null;
        }
      }
      filtered = null;
    }

  }
}
