class PcFormId extends PcFormElement
{
  constructor(props)
  {
    super(props);

    this.emailInput = React.createRef();
    this.label = React.createRef();

    if (props.listingFile!="")
    {
      this.currentMode = "id";
    }
    else {
      this.currentMode = "email";
    }

    this.listingData = null;
    this.currentListing = [];
    this.currentSelected = null;
  }

  componentDidMount()
  {
      this.readListingFile();
  }

  setFocus()
  {
    ReactDOM.findDOMNode(this.emailInput.current).focus();
  }

  readListingFile()
  {
    $.ajax({
      url: this.props.listingFile,
      type: "POST",
      timeout:5000,
      success: (data)=>{
        this.listingData = data;
        this.listingArray = this.createListingArray(data);
      },
      error: (xhr, status, err)=>{

      }
    });
  }

  createListingArray(data)
  {
    var a = [];
    data = data.replace(/(\r\n|\n|\r)/gm,"[line]");
    var lines = data.split('[line]');
    lines.forEach((item)=>{
        a.push(item.split(";"));
    });

    return a;
  }

  findStringInListing(str)
  {
    var a = [];

    this.listingArray.forEach((item, index)=>
    {
        if (this.findStringInItem(item, str))
        {
          a.push(item);
        }
    });

    a = a.slice(0,4);
    a.push(["", "Je ne suis pas dans la liste", "", ""]);

    return a;
  }

  findStringInItem(item, str)
  {
    str = str.trim();
    var astr = str.split(' ');
    var found = [];
    var foundcount = 0;

    for (let i=0; i<astr.length; i++)
    {
      item.forEach((subItem, index)=>{
        if (index>0 && !found[i] && subItem.toLowerCase().search(astr[i].toLowerCase())==0)
        {
          found[i] = true;
          foundcount++;
        }
      })
    }

    //if (str.length>=9) item.push(found);

    return (foundcount>=astr.length);
  }

  onInputChange()
  {
    this.props.answered(this.emailInput.current.value);
  }

  reset()
  {
    this.emailInput.current.value = "";
  }

  onKeyPressed(key)
  {
    if (key==PcConfig.keyboard.domain1 || key==PcConfig.keyboard.domain2)
    {
      if (this.emailInput.current.value.indexOf("@")==-1)
      {
        key = "@" + key;
      }

      this.emailInput.current.setSelectionRange(this.emailInput.current.value.length, this.emailInput.current.value.length);
    }

    let cpos = this.emailInput.current.selectionStart;
    let s1 = this.emailInput.current.value.substring(0,cpos);
    let s2 = this.emailInput.current.value.substring(cpos,this.emailInput.current.value.length);
    this.emailInput.current.setSelectionRange(cpos, cpos);

    if (key=="⌫")
    {
      if (this.currentSelected!=null)
      {
          this.currentSelected = null;
          cpos=0; s1=""; s2="";
          this.emailInput.current.value = "";
      }

      s1 = s1.substring(0, s1.length-1);
      this.emailInput.current.value = s1 + s2;
      this.emailInput.current.focus();
      if (cpos==0)
      {
        this.emailInput.current.setSelectionRange(0, 0);
      } else {
        this.emailInput.current.setSelectionRange(cpos-1, cpos-1);
      }
    }
    else if (key=="←")
    {
      this.emailInput.current.focus();
      if (cpos>0)
      {
        this.emailInput.current.setSelectionRange(cpos-1, cpos-1);
      }
    }
    else if (key=="→")
    {
      this.emailInput.current.focus();
      this.emailInput.current.setSelectionRange(cpos+1, cpos+1);
    }
    else
    {
      if (this.currentSelected!=null)
      {
          this.currentSelected = null;
          cpos=0; s1=""; s2="";
          this.emailInput.current.value = "";
      }
      this.emailInput.current.value = s1 + key + s2;
      this.emailInput.current.focus();
      this.emailInput.current.setSelectionRange(cpos+key.length, cpos+key.length);
    }

    this.onInputChange();

    if (this.emailInput.current.value.length>=3)
    {
      this.currentListing = this.findStringInListing(this.emailInput.current.value);
      this.forceUpdate();
    }
    else
    {
      this.currentListing=[];
      this.forceUpdate();
    }
  }

  onClickList(item)
  {
    if (item[0]!="")
    {
      this.emailInput.current.value = item[1] + " " + item[2];
      this.onKeyPressed('');
      this.currentListing=[];
      this.currentSelected = item;
      this.forceUpdate();
      console.log("item choose", item);
    }
    else {
      this.currentMode="email";
      this.forceUpdate();
      this.props.keyboardMustChange("email");
    }
  }

  validateEmail()
  {
    if (/^\w+([\.-]?\w+)*@\w+([\.-]?\w+)*(\.\w{2,8})+$/.test(this.emailInput.current.value))
    {
      return true;
    }
    else {
      return false;
    }
  }

  validate()
  {
    var validEmail = false;

    if (this.props.required)
    {
      validEmail = this.validateEmail();
    }
    else {
      if (this.emailInput.current.value.length>0)
      {
        validEmail = this.validateEmail();
      }
      else {
        validEmail = true;
      }
    }

    if (validEmail)
    {
      return "";
    }
    else {
      return this.props.errorMessage;
    }
  }

  changeLabel(text)
  {
    this.label.current.updateLabel(text);
  }

  render()
  {
    var idlist;

    if (this.currentMode=="id")
    {
     idlist = e(PcFormIdList,
        {
          onClick:(item)=>this.onClickList(item),
          width:"100%",
          list:this.currentListing,
          centered:this.props.centered ? "center" : "left",
        }
      );
    }
    else {
      idlist = null;
    }


    return e('div',
      {
        className:"pcemailcontainer",
        onClick:this.props.onClick,
        style:
        {
          textAlign:this.props.centered ? "center" : "left",
          transition:"all 0.25s ease-out",
        }
      },
        e(PcLabel,
          {
            ref:this.label,
            label:PcConfig.screen.infos.titleNoEmail,
            position:"relative",
            width:this.props.width,
            height:this.props.height,
            halign : this.props.centered ? "center" : "left",
            fontSize:PcConfig.scenario.fontSize,
            color: PcConfig.scenario.textColor,
            showDelay:"0,1s"
          }
        ),
        e('div', {style:{width:"100%", height:10*PcConfig.renderSize}}),
        e("input",
          {
            ref:this.emailInput,
            onChange:()=>this.onInputChange(),
            onInput:()=>this.onInputChange(),
            spellCheck:"false",
            style:
              {
                width:"98%",
                height:60*PcConfig.renderSize,
                fontFamily:PcConfig.scenario.fontFamily,
                fontSize:PcConfig.scenario.fontSize,
                color: PcConfig.scenario.textColor,
                borderStyle:"solid",
                borderWidth:PcConfig.scenario.buttonsBorderWidth/2,
                borderColor:PcConfig.scenario.textColor,
                borderRadius:12*PcConfig.renderSize,
                paddingLeft:10*PcConfig.renderSize,
                paddingRight:5*PcConfig.renderSize,
                textAlign:this.props.centered ? "center" : "left",
                background: "transparent",
                outline: "none",
              }
          }
        ),
        idlist,
        e('div', {style:{width:"100%", height:100*PcConfig.renderSize}}),
      );

  }

}
